@@ -3,7 +3,7 @@ import asyncio
 import copy
 import inspect
 import time
-from typing import Callable, Dict, Any, List
+from typing import Callable, Dict, Any, List, Union
 from toposort import toposort
 
 class Perform(object):
@@ -58,18 +58,30 @@ class Perform(object):
     @staticmethod
     def _fit_input_kwargs_from_fn(fn: Callable, input: Dict[str, Any]):
         fn_keys = dict(inspect.signature(fn).parameters)
-        return {k:v for k, v in input.items() if k in fn_keys}
+        kwargs = input if 'kwargs' in fn_keys else {k:v for k, v in input.items() if k in fn_keys}
+        args = list(input.values()) if 'args' in fn_keys else []
+        return args, kwargs
             
-    async def perform(self, input: Dict[str, Any] = {}) -> Any:
+    async def _perform(self, input: Dict[str, Any] = {}) -> Any:
         if self.should_verify:
             Piece.__verify_fn_inputs__(self.fn, actual_input=input)
+        
+        args, kwargs = self._fit_input_kwargs_from_fn(self.fn, input)
         output = await self.__timeit__(
-            async_method=self.fn(**self._fit_input_kwargs_from_fn(self.fn, input))
+            async_method=self.fn(*args, **kwargs)
         )
         if self.should_verify:
             Piece.__verify_fn_output__(self.fn, acutal_output=output)
         return output
 
+    async def perform(self, input: Union[Dict[str, Any], List[Dict[str, Any]]] = {}) -> Any:
+        if isinstance(input, list):
+            performers = [self._perform(input=inp) for inp in input]
+            result = await asyncio.gather(*performers, return_exceptions=True)
+        else:
+            result = await self._perform(input=input)
+        return result
+
     def copy(self):
         return copy.deepcopy(self)
 
@@ -105,10 +117,23 @@ class Piece(Perform):
         if expected_output_type == inspect._empty:
             raise TypeError(f"Output type cannot be untyped. In function {fn}, got '{expected_output_type}'")
 
-    async def perform(self, input: Dict[str, Any] = {}) -> Any:
-        return await super().perform({**input, **self.inputs})
+    async def _perform(self, input: Dict[str, Any]) -> Any:
+        return await super()._perform(input={**self.inputs, **input})
 
-class Competition(Perform):
+class PerformExecutor(object):
+
+    def __init__(self, execute: Callable, *args, **kwargs) -> None:
+        self.execute = execute
+        self.args = args
+        self.kwargs = kwargs
+
+class SimultaneousPerform(Perform):
+
+    def __init__(self, id: str, should_verify: bool, executor: PerformExecutor) -> None:
+        super().__init__(id, should_verify)
+        self.executor = executor
+
+class Competition(SimultaneousPerform):
 
     """
         A Competition is a race between two or more performers of whom is completed
@@ -116,15 +141,21 @@ class Competition(Perform):
         cancelled.
     """
 
-    def __init__(self, id: str, pieces: List[Perform], should_verify: bool = False) -> None:
-        super().__init__(id=id, should_verify=should_verify)
+    def __init__(
+        self, 
+        id: str, 
+        pieces: List[Perform], 
+        should_verify: bool = False, 
+        executor: PerformExecutor = PerformExecutor(asyncio.wait, return_when=asyncio.FIRST_COMPLETED),
+        ) -> None:
+        super().__init__(id=id, should_verify=should_verify, executor=executor)
         self.pieces = pieces
         self.outs = {}
 
-    async def fn(self, input: dict) -> Any:        
+    async def fn(self, **kwargs) -> Any:        
         tasks = []
         for piece in self.pieces:
-            task = asyncio.create_task(piece.perform(input))
+            task = asyncio.create_task(piece.perform(kwargs))
             task.id = piece.id
             tasks.append(task)
 
@@ -132,9 +163,10 @@ class Competition(Perform):
         result = None
         try:
             while tasks:
-                done, pending = await asyncio.wait(
+                done, pending = await self.executor.execute(
                     tasks,
-                    return_when=asyncio.FIRST_COMPLETED,
+                    *self.executor.args,
+                    **self.executor.kwargs,
                 )
 
                 for finished in done:
@@ -160,9 +192,6 @@ class Competition(Perform):
         except Exception as e:
             raise e
 
-    async def perform(self, input: Dict[str, Any]) -> Any:
-        return await super().perform(input={'input': input})
-
 class Dependency(object):
 
     """
@@ -177,14 +206,21 @@ class Dependency(object):
         self.dependent_on_id = dependent_on_id
         self.connected_on = connected_on
 
-class Composition(Perform):
+class Composition(SimultaneousPerform):
 
     """
         A Composition is a composition of pieces, sub classing Perform and running according to the piece's dependencies.
     """
 
-    def __init__(self, id: str, pieces: List[Piece], dependencies: List[Dependency] = [], should_verify: bool = False) -> None:
-        super().__init__(id=id, should_verify=should_verify)
+    def __init__(
+        self, 
+        id: str, 
+        pieces: List[Piece], 
+        dependencies: List[Dependency] = [], 
+        should_verify: bool = False,
+        executor: PerformExecutor = PerformExecutor(asyncio.gather, return_exceptions=True),
+        ) -> None:
+        super().__init__(id=id, should_verify=should_verify, executor=executor)
         self.pieces = pieces
         self.dependencies = dependencies
         self._pieces_dict = {piece.id: piece for piece in pieces}
@@ -233,7 +269,7 @@ class Composition(Perform):
             depend[dependency.piece_id].append((dependency.dependent_on_id, dependency.connected_on))
         return depend
 
-    async def fn(self, input: dict = {}) -> Any:
+    async def fn(self, **kwargs) -> Any:
         
         self.outs = {}
         for piece_paras in self.piece_running_order:
@@ -242,24 +278,21 @@ class Composition(Perform):
             for piece_key in piece_paras:
 
                 try:
-                    input_args = {**{}, **input}
+                    input_args = {**{}, **kwargs}
                     if piece_key in self._dependencies_dict:
                         for target_piece_key, piece_arg in self._dependencies_dict[piece_key]:
                             input_args.update({piece_arg: self.outs[target_piece_key]})
 
-                    tasks.append(asyncio.ensure_future(self._pieces_dict[piece_key].perform(input_args)))
+                    tasks.append(asyncio.create_task(self._pieces_dict[piece_key].perform(input_args)))
                 except Exception as e:
                     raise Exception(f"Could not construct task from piece function '{piece_key}' because of error: {e}")
 
-            piece_results = await asyncio.gather(*tasks, return_exceptions=True)
+            tasks += self.executor.args
+            piece_results = await self.executor.execute(*tasks, **self.executor.kwargs)
             for piece_result, piece_key in zip(piece_results, piece_paras):
                 self.outs[piece_key] = piece_result
 
-        return self.outs
-
-    async def perform(self, input: Dict[str, Any] = {}) -> Any:
-        result = await super().perform({'input': input})
-        return result[self.last_performer]
+        return self.outs[self.last_performer]
 
     def performer_results(self):
         return self.outs
\ No newline at end of file
