@@ -904,6 +904,34 @@ class TestFramework(IsolatedAsyncioTestCase):
         except Exception as e:
             self.fail(e)
 
+    async def test_competition_batches_of_pieces_will_pass(self):
+
+        async def fn0(d: dict) -> list:
+            return list(d.values())
+
+        async def fn1(d: dict) -> list:
+            await asyncio.sleep(1)
+            return list(d.keys())
+
+        competition = Competition(
+            id="my-competition",
+            pieces=[
+                Piece(id="contestent-0", fn=fn0),
+                Piece(id="contestent-1", fn=fn1),
+            ],
+        )
+
+        try:   
+            batches = [
+                {"d": {"a": 1, "b": 2, "c": 0}},
+                {"d": {"a": 1, "b": 2, "c": 1}},
+                {"d": {"a": 1, "b": 2, "c": 2}},
+            ]
+            result = await competition.perform(batches)
+            self.assertEqual(result, [[1,2,0], [1,2,1], [1,2,2]])
+        except Exception as e:
+            self.fail(e)
+
     async def test_competition_of_pieces_when_one_fail_will_pass(self):
 
         async def fn0(d: dict) -> list:
@@ -1398,4 +1426,81 @@ class TestFramework(IsolatedAsyncioTestCase):
         except:
             pass
         
-        
\ No newline at end of file
+    async def test_perform_piece_with_batching(self):
+
+        async def fn0(a: int, b: int) -> int:
+            return a + b
+
+        piece = Piece(
+            id="fn0",
+            fn=fn0,
+        )
+
+        batch_tests = [
+            {'a': 1, 'b': 1},
+            {'a': -1, 'b': 1},
+            {'a': 2, 'b': 1},
+            {'a': 1, 'b': 2},
+            {'a': 1, 'b': 3},
+            {'a': 1, 'b': 11},
+        ]
+        try:
+            results = await piece.perform(input=batch_tests)
+            self.assertEqual(results, [2, 0, 3, 3, 4, 12])
+        except Exception as e:
+            self.fail(e)
+        
+    async def test_perform_composition_with_batching(self):
+
+        async def add(a: int, b: int) -> int:
+            return a + b
+
+        async def mul(a: int, b: int) -> int:
+            return a * b * b
+
+        async def sub(a: int, b: int) -> int:
+            res = b-a
+            print(res)
+            return res
+
+        composition = Composition(
+            id="add-sub",
+            pieces=[
+                Piece(
+                    id="add",
+                    fn=add,
+                ),
+                Piece(
+                    id="mul",
+                    fn=mul,
+                ),
+                Piece(
+                    id="sub",
+                    fn=sub,
+                ),
+            ],
+            dependencies=[
+                Dependency(
+                    piece_id="sub",
+                    dependent_on_id="add",
+                    connected_on="a",
+                ),
+                Dependency(
+                    piece_id="sub",
+                    dependent_on_id="mul",
+                    connected_on="b",
+                ),
+            ],
+        )
+
+        batch_tests = [
+            {'a': -2, 'b': 55},
+            {'a': 1, 'b': 3},
+            {'a': 5, 'b': 11},
+        ]
+        try:
+            results = await composition.perform(input=batch_tests)
+            self.assertEqual(results, [-6103, 5, 589])
+        except Exception as e:
+            self.fail(e)
+
